﻿using System;
using Benday.FakesSample.Business;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.FakesSample.DataAccess;

namespace Benday.FakesSample.UnitTests
{
    [TestClass]
    public class PersonManagerFixture
    {
        private readonly DateTime EXPECTED_DEFAULT_DATE = 
            new DateTime(1900, 1, 1);

        [TestInitialize]
        public void OnTestInitialize()
        {
            
        }

        [TestCleanup]
        public void OnTestCleanup()
        {
            m_SystemUnderTest = null;
        }

        private PersonManager m_SystemUnderTest;
        public PersonManager SystemUnderTest
        {
            get
            {
                if (m_SystemUnderTest == null)
                {
                    m_SystemUnderTest = new PersonManager();
                }

                return m_SystemUnderTest;
            }
        }

        [TestMethod]
        public void WhenNewRecordIsCreatedButNotSavedCreateDateIs_19000101()
        {
            var person = SystemUnderTest.CreateNew();

            Assert.AreEqual<DateTime>(
                new DateTime(1900, 1, 1), 
                person.CreateDate, 
                "CreateDate was wrong.");
        }

        [TestMethod]
        public void WhenNewRecordIsCreatedButNotSavedLastModifiedIs_19000101()
        {
            var person = SystemUnderTest.CreateNew();

            Assert.AreEqual<DateTime>(
                EXPECTED_DEFAULT_DATE,
                person.LastModified,
                "LastModified was wrong.");
        }

        [TestMethod]
        public void WhenNewRecordIsCreatedButNotSavedIdIsLessThan1()
        {
            var person = SystemUnderTest.CreateNew();

            Assert.IsTrue(
                person.Id < 1, 
                "Person id should be less than 1.  Actual value was '{0}'.", 
                person.Id);            
        }

        private Person CreateAndPopulatePerson(
            string firstName, 
            string lastName, 
            string emailAddress, 
            string notes)
        {
            var person = SystemUnderTest.CreateNew();

            person.FirstName = firstName;
            person.LastName = lastName;
            person.Notes = notes;
            person.EmailAddress = emailAddress;

            return person;
        }

        [TestMethod]
        public void WhenNewRecordIsSavedCreateDateAndLastModifiedIsUpdated()
        {
            var person = 
                CreateAndPopulatePerson(
                    "Ben", "Day", 
                    "benday@benday.com", "yay! notes!!!");

            SystemUnderTest.Save(person);

            /*
            Assert.IsTrue(person.Id > 0, 
                "Person Id should be greater than 0.  Actual value was '{0}'.",
                person.Id);
            */

            Assert.AreNotEqual<DateTime>(
                EXPECTED_DEFAULT_DATE,
                person.LastModified,
                "LastModified was wrong.");

            Assert.AreNotEqual<DateTime>(
                EXPECTED_DEFAULT_DATE,
                person.CreateDate,
                "CreateDate was wrong.");
        }

        [TestMethod]
        public void WhenExisitingRecordIsSavedCreateDateAndLastModifiedIsUpdated()
        {
            var person =
                CreateAndPopulatePerson(
                    "Ben", "Day",
                    "benday@benday.com", "yay! notes!!!");

            SystemUnderTest.Save(person);

            DateTime originalCreateDate = person.CreateDate;
            DateTime originalLastModified = person.LastModified;

            person.Notes = "wheeeeeee!!! modified notes!!!";
            SystemUnderTest.Save(person);

            Assert.AreEqual<DateTime>(
                originalCreateDate,
                person.CreateDate,
                "Create date should not change.");

            Assert.AreNotEqual<DateTime>(
                originalLastModified,
                person.LastModified,
                "Last modified was not updated.");
        }        
    }
}
