﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Benday.FakesSample.DataAccess;
using Benday.FakesSample.Common;

namespace Benday.FakesSample.Business
{
    public class PersonManager
    {
        public PersonManager(
            IRepository<IPerson> repository,
            ILogger logger)
        {            
            if (repository == null)
            {
                throw new ArgumentNullException("repository", "repository is null.");
            }
            if (logger == null)
            {
                throw new ArgumentNullException("logger", "logger is null.");
            }

            _RepositoryInstance = repository;
            _LoggerInstance = logger;
        }

        private ILogger _LoggerInstance;
        private IRepository<IPerson> _RepositoryInstance;

        public IPerson CreateNew()
        {
            return new Person(
                0, 
                String.Empty, 
                String.Empty, 
                String.Empty, 
                String.Empty, 
                new DateTime(1900, 1, 1), 
                new DateTime(1900, 1, 1));
        }

        public void Save(IPerson saveThis)
        {
            try
            {
                if (saveThis.Id == 0)
                {
                    saveThis.CreateDate = DateTime.Now;
                    saveThis.LastModified = DateTime.Now;
                }
                else
                {
                    saveThis.LastModified = DateTime.Now;
                }

                _RepositoryInstance.Save(saveThis);
            }
            catch (SqlServerOutOfDiskSpaceException ex)
            {
                FixDiskSpaceProblem();
                // new Logger().WriteToLog(ex);
                _LoggerInstance.WriteToLog(ex);
                throw;
            }
            catch(Exception ex)
            {
                //new Logger().WriteToLog(ex);
                _LoggerInstance.WriteToLog(ex);
                throw;
            }
        }

        private void FixDiskSpaceProblem()
        {
            Console.WriteLine("Hey, somebody should really do something about that database.  It's outta disk space.");
        }               

        public IPerson LoadById(int id)
        {
            return _RepositoryInstance.GetById(id);
        }
    }
}
